#!/usr/bin/php
<?php
# =========================================================================
# This script resolves host fields from ip address fields in various tables
# It's called for tables: web, toolstart, sessionlog_metrics
#
# USAGE: ./xlogfix_dns_worker.php <database> <table> <startdate> <enddate>
#
# =========================================================================
# AUTHOR: Swaroop S, Purdue University
# Copyright (c) 2011 Purdue University
#
# BASED ON SIMILAR PERL SCRIPT BY: Swaroop S, Purdue University
# and PHP SCRIPT by Nicholas Kisseberth, Purdue University
# =========================================================================

error_reporting(E_ALL & ~E_NOTICE);
@ini_set('display_errors','1');

if(!defined('__DIR__')) {
    $fPos = strrpos(__FILE__, "/");
    define("__DIR__", substr(__FILE__, 0, $fPos) . "/");
}

require_once(__DIR__."/includes/hub_parameters.php");
require_once(__DIR__."/includes/db_connect.php");
require_once(__DIR__."/includes/func_misc.php");

$timeout = 1;
$db_hub = db_connect('db_hub');
$debug = 0;

if (!$_SERVER['argv'][1] || !$_SERVER['argv'][2] || !$_SERVER['argv'][3] || !$_SERVER['argv'][4])
{
    $msg = 'Usage: ' . $_SERVER['argv'][0] . ' <database> <table> <startdate> <enddate>'."\n";
    clean_exit($msg);
} else {
    $database = $_SERVER['argv'][1];
    $table = $_SERVER['argv'][2];
    $date = $_SERVER['argv'][3];
    $enddate = $_SERVER['argv'][4];
}

if ($database == 'hub') {
    $database = $hub_db;
} else if ($database == 'metrics') {
    $database = $metrics_db;
} else {
    $msg = 'Invalid database type'."\n";
    clean_exit($msg);
}

// set the correct column name: it's sessionlog_metrics.start but web.datetime and toolstart.datetime
if ($table == 'sessionlog_metrics') {
    $d_column = 'start';
} else {
    $d_column = 'datetime';
} 
// Let's not look up host=? records, for time savings
$sql = 'SELECT DISTINCT ip, host FROM '.$database.'.'.$table.' WHERE ip <> "" AND ip IS NOT NULL AND '.$d_column.' > "'.$date.'" and '.$d_column.' < "'.$enddate.'" AND (host = "" OR host IS NULL)';

if ($debug == 1) print "sql: $sql \n";
$result = mysqli_query($db_hub, $sql);
if($result) {
    $rowcount = mysqli_num_rows($result);
    if ($debug == 1) print "Row count: $rowcount\n";
    if ($rowcount > 0) {
        while($row = mysqli_fetch_row($result)) {
            if ($debug == 1) print "Looking up ".$row[0]." ".$timeout."\n";
            $host = xgethostbyaddr($row[0], $timeout);

            // if host lookup was unsuccessful, IP is returned
            if ($host == $row[0]) {
                $host = "?";
	        }

            if ($debug == 1 and $host<>"?") print " found $host\n";

            // Update table record if we found a host
            if($row[1] != "?" || $host != "?") {
		        // we are no longer updating host=? records, for time savings
                $sql_updt = 'UPDATE '.$database.'.'.$table.' SET host = '.dbquote($host).' WHERE (host = "" OR host IS NULL) AND ip = '.dbquote($row[0]);
                if ($debug == 1) print " updating $table record with $host\n";
                db_exec($db_hub, $sql_updt);
            }
        }
    }
} else {
    $msg = mysqli_error($db_hub).' while executing '.$sql."\n";
    clean_exit($msg);
}

db_close($db_hub);

?>
